@@ -2,7 +2,7 @@
  *
  * GNOME Radio
  *
- * Copyright (C) 2021, 2022, 2025  Aamot Research
+ * Copyright (C)  2015-2025  Aamot Research & Development
  *
  * Author: Ole Aamot <ole@aamot.org>
  *
@@ -38,6 +38,14 @@
 #include "gnome-radio-markers.h"
 #include "gnome-radio-player.h"
 
+#include "gnome-radio-signals.h"
+#include "gnome-radio-signals-crypto.h"
+
+SignalsConnection conn;
+
+GtkWidget *messages_view;
+GtkWidget *entry;
+
 #define N_COLS 2
 #define COL_ID 0
 #define COL_NAME 1
@@ -87,6 +95,59 @@ GStatBuf stats;
 
 ChamplainView *view;
 
+
+int crypto_init() {
+    return sodium_init() >= 0;
+}
+
+int generate_keypair(KeyPair *kp) {
+    return crypto_box_keypair(kp->public_key, kp->secret_key);
+}
+
+int encrypt_message(const unsigned char *msg, unsigned long long msg_len,
+                    const unsigned char *receiver_pub,
+                    const unsigned char *sender_secret,
+                    unsigned char *cipher, unsigned char *nonce) {
+    randombytes_buf(nonce, crypto_box_NONCEBYTES);
+    return crypto_box_easy(cipher, msg, msg_len, nonce, receiver_pub, sender_secret);
+}
+
+int decrypt_message(const unsigned char *cipher, unsigned long long cipher_len,
+                    const unsigned char *nonce,
+                    const unsigned char *sender_pub,
+                    const unsigned char *receiver_secret,
+                    unsigned char *msg) {
+    return crypto_box_open_easy(msg, cipher, cipher_len, nonce, sender_pub, receiver_secret);
+}
+
+int init_signals(SignalsConnection *conn) {
+    crypto_init();
+    generate_keypair(&conn->keypair);
+    memset(conn->peer_pub, 0, crypto_box_PUBLICKEYBYTES);
+    return 0;
+}
+
+int send_message(SignalsConnection *conn, const char *msg) {
+    unsigned char cipher[1024];
+    unsigned char nonce[crypto_box_NONCEBYTES];
+    encrypt_message((unsigned char*)msg, strlen(msg), conn->peer_pub, conn->keypair.secret_key, cipher, nonce);
+    // TODO: send cipher + nonce via websocket
+    return 0;
+}
+
+int receive_message(SignalsConnection *conn, const unsigned char *cipher, unsigned long long len, const unsigned char *nonce, char *msg) {
+    decrypt_message(cipher, len, nonce, conn->peer_pub, conn->keypair.secret_key, (unsigned char*)msg);
+    return 0;
+}
+
+static void send_button_clicked(GtkButton *button, gpointer user_data) {
+    const char *text = gtk_entry_get_text(GTK_ENTRY(entry));
+    send_message(&conn, text);
+    gtk_text_buffer_insert_at_cursor(gtk_text_view_get_buffer(GTK_TEXT_VIEW(messages_view)), text, -1);
+    gtk_text_buffer_insert_at_cursor(gtk_text_view_get_buffer(GTK_TEXT_VIEW(messages_view)), "\n", -1);
+    gtk_entry_set_text(GTK_ENTRY(entry), "");
+}
+
 gchar *
 str_channels (GNOMERadioChannels type) {
   gchar *channels;
@@ -573,12 +634,9 @@ gnome_radio_window_cb (GtkApplication *app,
 	search = gtk_tool_button_new(gtk_image_new_from_icon_name(NULL, GTK_ICON_SIZE_BUTTON), _("Search"));
 	gtk_tool_item_set_is_important(GTK_TOOL_ITEM(search), TRUE);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM(search), 1);
-	gtk_widget_show (GTK_WIDGET(search));
+	gtk_widget_show_all (GTK_WIDGET(search));
 	gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM(search), _("Search Internet Radio Station"));
 	g_signal_connect(search, "clicked", G_CALLBACK (search_station), GTK_WINDOW (window));
-#endif
-
-#if 0
 	prev = gtk_tool_button_new(gtk_image_new_from_icon_name(NULL, GTK_ICON_SIZE_BUTTON), _("Prev"));
 	gtk_tool_item_set_is_important(GTK_TOOL_ITEM(prev), TRUE);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM(prev), 5);
@@ -623,7 +681,7 @@ gnome_radio_window_cb (GtkApplication *app,
 	gtk_container_add (GTK_CONTAINER(window), GTK_WIDGET(grid));
 	g_signal_connect (window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
 #endif
-	gtk_window_set_title (GTK_WINDOW(window), _("Radio 60.0 - https://www.gnomeradio.org/ - https://wiki.gnome.org/Apps/Radio"));
+	gtk_window_set_title (GTK_WINDOW(window), _("Radio 61.0 - https://www.gnomeradio.org/ - https://wiki.gnome.org/Apps/Radio"));
 	gtk_window_set_default_size (GTK_WINDOW(window), 800, 600);
 	gtk_window_maximize (GTK_WINDOW (window));
 	gnome_radio_app = create_gnome_radio_app();
@@ -807,11 +865,12 @@ static void activate(GtkApplication *app, gpointer user_data)
 	// if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
 	//	return 1;
 	// gtk_init (&argc, &argv);
+
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	/* give the window a 10px wide border */
 	gtk_container_set_border_width (GTK_CONTAINER (window), 10);
 	/* give it the title */
-	gtk_window_set_title (GTK_WINDOW (window), _("Radio 60.0 - https://www.gnomeradio.org/ - https://wiki.gnome.org/Apps/Radio"));
+	gtk_window_set_title (GTK_WINDOW (window), _("Radio 61.0 - https://www.gnomeradio.org/ - https://wiki.gnome.org/Apps/Radio"));
 	/* Connect the destroy event of the window with our on_destroy function
 	 * When the window is about to be destroyed we get a notificaiton and
 	 * stop the main GTK loop
@@ -843,7 +902,6 @@ static void activate(GtkApplication *app, gpointer user_data)
 	clutter_actor_set_x_align (scale, CLUTTER_ACTOR_ALIGN_START);
 
 	clutter_actor_set_y_align (scale, CLUTTER_ACTOR_ALIGN_END);
-	clutter_actor_add_child (CLUTTER_ACTOR (view), scale);
 
 	license_actor = champlain_view_get_license_actor (view);
 	champlain_license_set_extra_text (license_actor, "Public Internet Radio");
@@ -1068,6 +1126,7 @@ int main (int argc, char **argv)
   player = gst_player_new (NULL, gst_player_g_main_context_signal_dispatcher_new(NULL));
   gnome_radio_player_new(player, "https://playerservices.streamtheworld.com/api/livestream-redirect/CSPANRADIO.mp3");
   gst_player_play(player);
+
   app = gtk_application_new ("org.gnome.radio", G_APPLICATION_FLAGS_NONE);
 	g_signal_connect (app, "activate", G_CALLBACK (activate), NULL);
 	status = g_application_run (G_APPLICATION (app), argc, argv);
