@@ -791,10 +791,9 @@ main (int argc,
 	clutter_actor_set_x_align (scale, CLUTTER_ACTOR_ALIGN_START);
 
 	clutter_actor_set_y_align (scale, CLUTTER_ACTOR_ALIGN_END);
-	clutter_actor_add_child (CLUTTER_ACTOR (view), scale);
 
 	license_actor = champlain_view_get_license_actor (view);
-	champlain_license_set_extra_text (license_actor, "Public Internet Radio");
+	champlain_license_set_extra_text (license_actor, "Internet Talk Radio");
 	champlain_view_center_on (CHAMPLAIN_VIEW (view), 44.5079609, -73.1534229);
 	layer = create_marker_layer (view, &path);
 	champlain_view_add_layer (view, CHAMPLAIN_LAYER (path));
@@ -993,6 +992,10 @@ main (int argc,
 
 	gst_init(&argc, &argv);
 
+	  player = gst_player_new (NULL, gst_player_g_main_context_signal_dispatcher_new(NULL));
+	  gtk_internet_radio_locator_player_new(player, "https://playerservices.streamtheworld.com/api/livestream-redirect/CSPANRADIO.mp3");
+  gst_player_play(player);
+
 	/* start the main loop */
 	gtk_main ();
 
